@@ -93,6 +93,7 @@ VLC_BOOTSTRAP_ARGS="\
     --disable-aribb25 \
     --disable-mpg123 \
     --enable-libdsm \
+    --enable-libarchive \
 "
 
 ###########################
@@ -107,7 +108,6 @@ VLC_CONFIGURE_ARGS="\
     --enable-avcodec \
     --enable-opus \
     --enable-opensles \
-    --enable-android-surface \
     --enable-mkv \
     --enable-taglib \
     --enable-dvbpsi \
@@ -281,11 +281,11 @@ REL=$(grep -o '^r[0-9]*.*' $ANDROID_NDK/RELEASE.TXT 2>/dev/null|cut -b2-)
 case "$REL" in
     10*)
         if [ "${HAVE_64}" = 1 ];then
-            GCCVER=4.9
             ANDROID_API=android-21
+            GCCVER=4.9
         else
-            GCCVER=4.8
             ANDROID_API=android-9
+            GCCVER=4.8
         fi
     ;;
     *)
@@ -376,6 +376,9 @@ fi
 EXTRA_CFLAGS="${EXTRA_CFLAGS} -I${ANDROID_NDK}/sources/cxx-stl/gnu-libstdc++/${GCCVER}/include"
 EXTRA_CFLAGS="${EXTRA_CFLAGS} -I${ANDROID_NDK}/sources/cxx-stl/gnu-libstdc++/${GCCVER}/libs/${ANDROID_ABI}/include"
 
+# XXX: remove when ndk C++11 is updated
+EXTRA_CXXFLAGS="-D__STDC_FORMAT_MACROS=1 -D__STDC_CONSTANT_MACROS=1 -D__STDC_LIMIT_MACROS=1"
+
 CPPFLAGS="-I${ANDROID_NDK}/sources/cxx-stl/gnu-libstdc++/${GCCVER}/include -I${ANDROID_NDK}/sources/cxx-stl/gnu-libstdc++/${GCCVER}/libs/${ANDROID_ABI}/include"
 
 #################
@@ -503,6 +506,7 @@ if [ "${CHROME_OS}" = "1" ];then
     # chrome OS doesn't have eventfd
     export ac_cv_func_eventfd=no
     export ac_cv_header_sys_eventfd_h=no
+    export ac_cv_func_pipe2=no
 fi
 
 if [ ${ANDROID_API} = "android-21" ] ; then
@@ -518,7 +522,7 @@ fi
 if [ ! -e ./config.h -o "$RELEASE" = 1 ]; then
 CPPFLAGS="$CPPFLAGS" \
 CFLAGS="$CFLAGS ${EXTRA_CFLAGS}" \
-CXXFLAGS="$CFLAGS" \
+CXXFLAGS="$CFLAGS ${EXTRA_CXXFLAGS}" \
 LDFLAGS="$LDFLAGS" \
 CC="${CROSS_COMPILE}gcc --sysroot=${SYSROOT}" \
 CXX="${CROSS_COMPILE}g++ --sysroot=${SYSROOT} -D__cpp_static_assert=200410" \
@@ -551,9 +555,9 @@ cd $SRC_DIR
 # libVLC modules #
 ##################
 
-REDIFINED_VLC_MODULES_DIR=$SRC_DIR/.modules/${VLC_BUILD_DIR}
-rm -rf ${REDIFINED_VLC_MODULES_DIR}
-mkdir -p ${REDIFINED_VLC_MODULES_DIR}
+REDEFINED_VLC_MODULES_DIR=$SRC_DIR/.modules/${VLC_BUILD_DIR}
+rm -rf ${REDEFINED_VLC_MODULES_DIR}
+mkdir -p ${REDEFINED_VLC_MODULES_DIR}
 
 echo "Generating static module list"
 blacklist_regexp=
@@ -581,35 +585,45 @@ VLC_MODULES=$(find_modules vlc/$VLC_BUILD_DIR/modules)
 DEFINITION="";
 BUILTINS="const void *vlc_static_modules[] = {\n";
 for file in $VLC_MODULES; do
-    outfile=${REDIFINED_VLC_MODULES_DIR}/`basename $file`
+    outfile=${REDEFINED_VLC_MODULES_DIR}/`basename $file`
     name=`echo $file | sed 's/.*\.libs\/lib//' | sed 's/_plugin\.a//'`;
     symbols=$("${CROSS_COMPILE}nm" -g $file)
 
-    # assure that all modules have differents entry symbols
+    # assure that all modules have differents symbol names
     entry=$(get_symbol "$symbols" _)
     copyright=$(get_symbol "$symbols" copyright)
     license=$(get_symbol "$symbols" license)
-    cat <<EOF > ${REDIFINED_VLC_MODULES_DIR}/syms
+    cat <<EOF > ${REDEFINED_VLC_MODULES_DIR}/syms
+AccessOpen AccessOpen__$name
+AccessClose AccessClose__$name
+StreamOpen StreamOpen__$name
+StreamClose StreamClose__$name
+DemuxOpen DemuxOpen__$name
+DemuxClose DemuxClose__$name
+OpenFilter OpenFilter__$name
+CloseFilter CloseFilter__$name
+Open Open__$name
+Close Close__$name
 $entry vlc_entry__$name
 $copyright vlc_entry_copyright__$name
 $license vlc_entry_license__$name
 EOF
-    ${CROSS_COMPILE}objcopy --redefine-syms ${REDIFINED_VLC_MODULES_DIR}/syms $file $outfile
+    ${CROSS_COMPILE}objcopy --redefine-syms ${REDEFINED_VLC_MODULES_DIR}/syms $file $outfile
     checkfail "objcopy failed"
 
     DEFINITION=$DEFINITION"int vlc_entry__$name (int (*)(void *, void *, int, ...), void *);\n";
     BUILTINS="$BUILTINS vlc_entry__$name,\n";
 done;
 BUILTINS="$BUILTINS NULL\n};\n"; \
-printf "/* Autogenerated from the list of modules */\n$DEFINITION\n$BUILTINS\n" > libvlc/jni/libvlcjni.h
-rm ${REDIFINED_VLC_MODULES_DIR}/syms
+printf "/* Autogenerated from the list of modules */\n$DEFINITION\n$BUILTINS\n" > libvlc/jni/libvlcjni-modules.h
+rm ${REDEFINED_VLC_MODULES_DIR}/syms
 
 ###############################
 # NDK-Build for libvlcjni.so  #
 ###############################
 
 LIBVLC_LIBS="libvlcjni"
-VLC_MODULES=$(find_modules ${REDIFINED_VLC_MODULES_DIR})
+VLC_MODULES=$(find_modules ${REDEFINED_VLC_MODULES_DIR})
 VLC_SRC_DIR="$SRC_DIR/vlc"
 ANDROID_SYS_HEADERS="$SRC_DIR/android-headers"
 VLC_CONTRIB="$VLC_SRC_DIR/contrib/$TARGET_TUPLE"
@@ -647,6 +661,7 @@ $ANDROID_NDK/ndk-build -C libvlc \
     TARGET_TUPLE=$TARGET_TUPLE \
     HAVE_64=${HAVE_64} \
     NDK_PROJECT_PATH=jni \
+    NDK_TOOLCHAIN_VERSION=${GCCVER} \
     NDK_DEBUG=${NDK_DEBUG}
 
 checkfail "ndk-build failed"
