@@ -39,21 +39,28 @@ typedef struct java_event java_event;
 
 struct vlcjni_object
 {
+    /* Pointer to parent libvlc: NULL if the VLCObject is a LibVLC */
     libvlc_instance_t *p_libvlc;
+
+    /* Current pointer to native vlc object */
     union {
+        libvlc_instance_t *p_libvlc;
         libvlc_media_t *p_m;
         libvlc_media_list_t *p_ml;
         libvlc_media_discoverer_t *p_md;
+        libvlc_media_player_t *p_mp;
     } u;
-    vlcjni_object_owner *p_owner; // used by vlcobject
-    vlcjni_object_sys *p_sys; // used by media, medialist, mediadiscoverer...
+    /* Used by vlcobject */
+    vlcjni_object_owner *p_owner;
+    /* Used by media, medialist, mediadiscoverer... */
+    vlcjni_object_sys *p_sys;
 };
 
 struct java_event
 {
     jint type;
     jlong arg1;
-    jlong arg2;
+    jfloat arg2;
 };
 
 /* event manager callback dispatched to native struct implementing a
@@ -66,12 +73,10 @@ typedef bool (*event_cb)(vlcjni_object *p_obj, const libvlc_event_t *p_ev,
 vlcjni_object *VLCJniObject_getInstance(JNIEnv *env, jobject thiz);
 
 vlcjni_object *VLCJniObject_newFromJavaLibVlc(JNIEnv *env, jobject thiz,
-                                              jobject libVlc,
-                                              const char **pp_error);
+                                              jobject libVlc);
 
 vlcjni_object *VLCJniObject_newFromLibVlc(JNIEnv *env, jobject thiz,
-                                          libvlc_instance_t *p_libvlc,
-                                          const char **pp_error);
+                                          libvlc_instance_t *p_libvlc);
 
 void VLCJniObject_release(JNIEnv *env, jobject thiz, vlcjni_object *p_obj);
 
