@@ -20,7 +20,7 @@
 
 package org.videolan.vlc;
 
-import android.content.Context;
+import android.net.Uri;
 import android.os.Environment;
 import android.os.Handler;
 import android.os.Message;
@@ -29,6 +29,7 @@ import android.util.Log;
 
 import org.videolan.libvlc.LibVLC;
 import org.videolan.libvlc.Media;
+import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.libvlc.util.Extensions;
 import org.videolan.vlc.gui.audio.AudioBrowserListAdapter;
 import org.videolan.vlc.interfaces.IBrowser;
@@ -200,7 +201,7 @@ public class MediaLibrary {
                 continue;
             playList = new AudioBrowserListAdapter.ListItem(playlistName, null, null, false);
             for (String track : items){
-                playList.mMediaList.add(new MediaWrapper(track));
+                playList.mMediaList.add(new MediaWrapper(AndroidUtil.LocationToUri(track)));
             }
             playlistItems.add(playList);
         }
@@ -286,7 +287,6 @@ public class MediaLibrary {
                 while (!directories.isEmpty()) {
                     File dir = directories.pop();
                     String dirPath = dir.getAbsolutePath();
-                    File[] f = null;
 
                     // Skip some system folders
                     if (dirPath.startsWith("/proc/") || dirPath.startsWith("/sys/") || dirPath.startsWith("/dev/"))
@@ -312,17 +312,15 @@ public class MediaLibrary {
                     // Filter the extensions and the folders
                     try {
                         String[] files = dir.list();
-                        File file;
-                        if (files != null){
-                            for (String fileName : files){
-                                file = new File(dirPath, fileName);
+                        if (files != null) {
+                            for (String fileName : files) {
+                                File file = new File(dirPath, fileName);
                                 if (mediaFileFilter.accept(file)){
                                     if (file.isFile())
                                         mediaToScan.add(file);
                                     else if (file.isDirectory())
                                         directories.push(file);
                                 }
-                                file = null;
                             }
                         }
                     } catch (Exception e){
@@ -337,14 +335,14 @@ public class MediaLibrary {
                 }
 
                 //Remove ignored files
-                HashSet<String> mediasToRemove = new HashSet<String>();
+                HashSet<Uri> mediasToRemove = new HashSet<Uri>();
                 String path;
                 outloop:
                 for (Map.Entry<String, MediaWrapper> entry : existingMedias.entrySet()){
                     path = entry.getKey();
                     for (String dirPath : dirsToIgnore) {
                         if (path.startsWith(dirPath)) {
-                            mediasToRemove.add(path);
+                            mediasToRemove.add(entry.getValue().getUri());
                             mItemList.remove(existingMedias.get(path));
                             continue outloop;
                         }
@@ -354,7 +352,7 @@ public class MediaLibrary {
 
                 // Process the stacked items
                 for (File file : mediaToScan) {
-                    String fileURI = LibVLC.PathToURI(file.getPath());
+                    String fileURI = AndroidUtil.FileToUri(file).toString();
                     if (mBrowser != null && mBrowser.get() != null)
                         mBrowser.get().sendTextInfo(file.getName(), count,
                                 mediaToScan.size());
@@ -374,16 +372,17 @@ public class MediaLibrary {
                     } else {
                         mItemListLock.writeLock().lock();
                         // create new media item
-                        final Media media = new Media(libVlcInstance, fileURI);
+                        final Media media = new Media(libVlcInstance, Uri.parse(fileURI));
                         media.parse();
-                        media.release();
                         /* skip files with .mod extension and no duration */
                         if ((media.getDuration() == 0 || (media.getTrackCount() != 0 && TextUtils.isEmpty(media.getTrack(0).codec))) &&
                             fileURI.endsWith(".mod")) {
                             mItemListLock.writeLock().unlock();
+                            media.release();
                             continue;
                         }
                         MediaWrapper mw = new MediaWrapper(media);
+                        media.release();
                         mw.setLastModified(file.lastModified());
                         mItemList.add(mw);
                         // Add this item to database
@@ -407,7 +406,7 @@ public class MediaLibrary {
                     for (String fileURI : addedLocations) {
                         existingMedias.remove(fileURI);
                     }
-                    mediaDatabase.removeMedias(existingMedias.keySet());
+                    mediaDatabase.removeMediaWrappers(existingMedias.values());
 
                     /*
                      * In case of file matching path of a folder from another removable storage
@@ -432,7 +431,7 @@ public class MediaLibrary {
                 }
             }
         }
-    };
+    }
 
     private Handler restartHandler = new RestartHandler(this);
 
