@@ -29,9 +29,13 @@ import android.content.Intent;
 import android.content.IntentFilter;
 import android.content.SharedPreferences;
 import android.os.Bundle;
+import android.os.Handler;
+import android.os.Message;
 import android.preference.PreferenceManager;
+import android.support.v4.app.FragmentManager;
 import android.support.v7.app.ActionBar;
 import android.support.v7.app.AppCompatActivity;
+import android.support.v4.app.Fragment;
 import android.support.v7.widget.Toolbar;
 import android.view.LayoutInflater;
 import android.view.View;
@@ -39,25 +43,31 @@ import android.view.ViewGroup;
 import android.widget.TextView;
 
 import org.videolan.vlc.BuildConfig;
+import org.videolan.vlc.MediaLibrary;
+import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
-import org.videolan.vlc.audio.AudioServiceController;
 import org.videolan.vlc.gui.audio.AudioPlayer;
+import org.videolan.vlc.gui.browser.MediaBrowserFragment;
+import org.videolan.vlc.interfaces.IRefreshable;
 import org.videolan.vlc.util.Util;
+import org.videolan.vlc.util.WeakHandler;
 import org.videolan.vlc.widget.HackyDrawerLayout;
-import org.videolan.vlc.widget.SlidingPaneLayout;
+import com.android.widget.SlidingPaneLayout;
 
-public class AudioPlayerContainerActivity extends AppCompatActivity {
+public class AudioPlayerContainerActivity extends AppCompatActivity implements PlaybackService.Client.Callback  {
 
+    public static final String TAG = "VLC/AudioPlayerContainerActivity";
     public static final String ACTION_SHOW_PLAYER = "org.videolan.vlc.gui.ShowPlayer";
 
     protected ActionBar mActionBar;
     protected Toolbar mToolbar;
     protected AudioPlayer mAudioPlayer;
-    protected AudioServiceController mAudioController;
     protected SlidingPaneLayout mSlidingPane;
     protected View mAudioPlayerFilling;
     protected SharedPreferences mSettings;
     protected ViewGroup mRootContainer;
+    private final PlaybackServiceActivity.Helper mHelper = new PlaybackServiceActivity.Helper(this, this);
+    protected PlaybackService mService;
 
     @Override
     protected void onCreate(Bundle savedInstanceState) {
@@ -65,6 +75,10 @@ public class AudioPlayerContainerActivity extends AppCompatActivity {
         mSettings = PreferenceManager.getDefaultSharedPreferences(this);
         /* Theme must be applied before super.onCreate */
         applyTheme();
+
+        /* Set up the audio player */
+        mAudioPlayer = new AudioPlayer();
+
         super.onCreate(savedInstanceState);
     }
 
@@ -80,10 +94,7 @@ public class AudioPlayerContainerActivity extends AppCompatActivity {
         mSlidingPane.setPanelSlideListener(mPanelSlideListener);
         mAudioPlayerFilling = findViewById(R.id.audio_player_filling);
 
-        /* Set up the audio player */
-        mAudioPlayer = new AudioPlayer();
         mAudioPlayer.setUserVisibleHint(false);
-        mAudioController = AudioServiceController.getInstance();
 
         getSupportFragmentManager().beginTransaction()
                 .replace(R.id.audio_player, mAudioPlayer)
@@ -93,6 +104,14 @@ public class AudioPlayerContainerActivity extends AppCompatActivity {
     @Override
     protected void onStart() {
         super.onStart();
+        mHelper.onStart();
+
+        //Handle external storage state
+        IntentFilter storageFilter = new IntentFilter();
+        storageFilter.addAction(Intent.ACTION_MEDIA_MOUNTED);
+        storageFilter.addAction(Intent.ACTION_MEDIA_UNMOUNTED);
+        storageFilter.addDataScheme("file");
+        registerReceiver(storageReceiver, storageFilter);
 
         /* Prepare the progressBar */
         IntentFilter filter = new IntentFilter();
@@ -103,23 +122,11 @@ public class AudioPlayerContainerActivity extends AppCompatActivity {
     @Override
     protected void onStop() {
         super.onStop();
+        unregisterReceiver(storageReceiver);
         try {
             unregisterReceiver(messageReceiver);
         } catch (IllegalArgumentException e) {}
-    }
-
-    @Override
-    protected void onResume() {
-        super.onResume();
-        mAudioController.addAudioPlayer(mAudioPlayer);
-        AudioServiceController.getInstance().bindAudioService(this);
-    }
-
-    @Override
-    protected void onPause() {
-        super.onPause();
-        mAudioController.removeAudioPlayer(mAudioPlayer);
-        AudioServiceController.getInstance().unbindAudioService(this);
+        mHelper.onStop();
     }
 
     private void applyTheme() {
@@ -129,6 +136,23 @@ public class AudioPlayerContainerActivity extends AppCompatActivity {
         }
     }
 
+    public void updateLib() {
+        FragmentManager fm = getSupportFragmentManager();
+        Fragment current = fm.findFragmentById(R.id.fragment_placeholder);
+        if (current != null && current instanceof IRefreshable) {
+            ((IRefreshable) current).refresh();
+        } else
+            MediaLibrary.getInstance().loadMediaItems();
+        Fragment fragment = fm.findFragmentByTag(SidebarAdapter.SidebarEntry.ID_AUDIO);
+        if (fragment != null && !fragment.equals(current)) {
+            ((MediaBrowserFragment)fragment).clear();
+        }
+        fragment = fm.findFragmentByTag(SidebarAdapter.SidebarEntry.ID_VIDEO);
+        if (fragment != null && !fragment.equals(current)) {
+            ((MediaBrowserFragment)fragment).clear();
+        }
+    }
+
     /**
      * Show a tip view.
      * @param layoutId the layout of the tip view
@@ -278,4 +302,64 @@ public class AudioPlayerContainerActivity extends AppCompatActivity {
 
     protected void onPanelOpenedUiSet() {}
 
+    private void stopBackgroundTasks() {
+        MediaLibrary ml = MediaLibrary.getInstance();
+        if (ml.isWorking())
+            ml.stop();
+    }
+
+    private final BroadcastReceiver storageReceiver = new BroadcastReceiver() {
+        @Override
+        public void onReceive(Context context, Intent intent) {
+            String action = intent.getAction();
+
+            if (action.equalsIgnoreCase(Intent.ACTION_MEDIA_MOUNTED)) {
+                mStorageHandlerHandler.sendEmptyMessage(ACTION_MEDIA_MOUNTED);
+            } else if (action.equalsIgnoreCase(Intent.ACTION_MEDIA_UNMOUNTED)) {
+                mStorageHandlerHandler.sendEmptyMessageDelayed(ACTION_MEDIA_UNMOUNTED, 100);
+            }
+        }
+    };
+
+    Handler mStorageHandlerHandler = new StorageHandler(this);
+
+    private static final int ACTION_MEDIA_MOUNTED = 1337;
+    private static final int ACTION_MEDIA_UNMOUNTED = 1338;
+
+    private static class StorageHandler extends WeakHandler<AudioPlayerContainerActivity> {
+
+        public StorageHandler(AudioPlayerContainerActivity owner) {
+            super(owner);
+        }
+
+        @Override
+        public void handleMessage(Message msg) {
+            super.handleMessage(msg);
+
+            switch (msg.what){
+                case ACTION_MEDIA_MOUNTED:
+                    removeMessages(ACTION_MEDIA_UNMOUNTED);
+                    getOwner().updateLib();
+                    break;
+                case ACTION_MEDIA_UNMOUNTED:
+                    getOwner().stopBackgroundTasks();
+                    getOwner().updateLib();
+                    break;
+            }
+        }
+    }
+
+    public PlaybackServiceActivity.Helper getHelper() {
+        return mHelper;
+    }
+
+    @Override
+    public void onConnected(PlaybackService service) {
+        mService = service;
+    }
+
+    @Override
+    public void onDisconnected() {
+        mService = null;
+    }
 }
