@@ -56,19 +56,19 @@ import android.widget.ListView;
 import android.widget.ProgressBar;
 import android.widget.TextView;
 
-import org.videolan.libvlc.LibVlcUtil;
+import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.vlc.BuildConfig;
 import org.videolan.vlc.MediaDatabase;
 import org.videolan.vlc.MediaLibrary;
+import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
-import org.videolan.vlc.audio.AudioService;
 import org.videolan.vlc.gui.SidebarAdapter.SidebarEntry;
 import org.videolan.vlc.gui.audio.AudioBrowserFragment;
 import org.videolan.vlc.gui.browser.BaseBrowserFragment;
 import org.videolan.vlc.gui.browser.MediaBrowserFragment;
 import org.videolan.vlc.gui.browser.NetworkBrowserFragment;
-import org.videolan.vlc.gui.browser.StorageBrowserFragment;
+import org.videolan.vlc.gui.network.MRLPanelFragment;
 import org.videolan.vlc.gui.video.VideoGridFragment;
 import org.videolan.vlc.gui.video.VideoListAdapter;
 import org.videolan.vlc.gui.video.VideoPlayerActivity;
@@ -202,7 +202,6 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
         reloadPreferences();
     }
 
-
     @Override
     protected void onPostCreate(Bundle savedInstanceState) {
         super.onPostCreate(savedInstanceState);
@@ -221,8 +220,8 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
         super.onResume();
 
         /* FIXME: this is used to avoid having MainActivity twice in the backstack */
-        if (getIntent().hasExtra(AudioService.START_FROM_NOTIFICATION))
-            getIntent().removeExtra(AudioService.START_FROM_NOTIFICATION);
+        if (getIntent() != null && getIntent().hasExtra(PlaybackService.START_FROM_NOTIFICATION))
+            getIntent().removeExtra(PlaybackService.START_FROM_NOTIFICATION);
 
 
         /* Load media items from database and storage */
@@ -239,12 +238,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
         // Figure out if currently-loaded fragment is a top-level fragment.
         Fragment current = getSupportFragmentManager()
                 .findFragmentById(R.id.fragment_placeholder);
-        boolean found = false;
-        if(current != null) {
-            found = SidebarAdapter.sidebarFragments.contains(current.getTag());
-        } else {
-            found = true;
-        }
+        boolean found = (current == null) || SidebarAdapter.sidebarFragments.contains(current.getTag());
 
         /**
          * Restore the last view.
@@ -258,11 +252,11 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
          * It will try to remove() the currently displayed fragment
          * (i.e. tracks) and replace it with a blank screen. (stuck menu bug)
          */
-        if(current == null || (!current.getTag().equals(mCurrentFragment) && found)) {
+        if (current == null || (!current.getTag().equals(mCurrentFragment) && found)) {
             Log.d(TAG, "Reloading displayed fragment");
-            if(mCurrentFragment == null)
+            if (mCurrentFragment == null)
                 mCurrentFragment = "video";
-            if(!SidebarAdapter.sidebarFragments.contains(mCurrentFragment)) {
+            if (!SidebarAdapter.sidebarFragments.contains(mCurrentFragment)) {
                 Log.d(TAG, "Unknown fragment \"" + mCurrentFragment + "\", resetting to video");
                 mCurrentFragment = "video";
             }
@@ -395,7 +389,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
         MenuInflater inflater = getMenuInflater();
         inflater.inflate(R.menu.media_library, menu);
 
-        if (LibVlcUtil.isFroyoOrLater()) {
+        if (AndroidUtil.isFroyoOrLater()) {
             SearchManager searchManager =
                     (SearchManager) VLCApplication.getAppContext().getSystemService(Context.SEARCH_SERVICE);
             mSearchView = (SearchView) MenuItemCompat.getActionView(menu.findItem(R.id.ml_menu_search));
@@ -452,7 +446,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
             item = menu.findItem(R.id.ml_menu_save);
             item.setVisible(true);
             String mrl = ((BaseBrowserFragment)current).mMrl;
-            item.setIcon(MediaDatabase.getInstance().networkFavExists(mrl) ?
+            item.setIcon(MediaDatabase.getInstance().networkFavExists(Uri.parse(mrl)) ?
                     R.drawable.ic_menu_bookmark_w :
                     R.drawable.ic_menu_bookmark_outline_w);
         } else
@@ -505,12 +499,12 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
             // Restore last playlist
             case R.id.ml_menu_last_playlist:
                 if (current instanceof AudioBrowserFragment) {
-                    Intent i = new Intent(AudioService.ACTION_REMOTE_LAST_PLAYLIST);
+                    Intent i = new Intent(PlaybackService.ACTION_REMOTE_LAST_PLAYLIST);
                     sendBroadcast(i);
                 } else if (current instanceof VideoGridFragment) {
-                    String location = Uri.decode(mSettings.getString(PreferencesActivity.VIDEO_LAST, null));
-                    if (location != null)
-                        VideoPlayerActivity.start(this, location);
+                    final Uri uri = Uri.parse(mSettings.getString(PreferencesActivity.VIDEO_LAST, null));
+                    if (uri != null)
+                        VideoPlayerActivity.start(this, uri);
                 }
                 break;
             case android.R.id.home:
@@ -588,7 +582,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
         View playPause = findViewById(R.id.header_play_pause);
 
         if (!idIsEmpty) {
-            View list = null;
+            View list;
             int pane = mSlidingPane.getState();
 
             if (parentView == null)
@@ -651,7 +645,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
             v.setNextFocusDownId(mActionBarIconId);
             v.setNextFocusLeftId(mActionBarIconId);
             v.setNextFocusRightId(R.id.ml_menu_search);
-            if (LibVlcUtil.isHoneycombOrLater())
+            if (AndroidUtil.isHoneycombOrLater())
                 v.setNextFocusForwardId(mActionBarIconId);
             if (findViewById(R.id.ml_menu_search) != null)
                 findViewById(R.id.ml_menu_search).setNextFocusLeftId(mActionBarIconId);
@@ -714,7 +708,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
                     break;
             }
         }
-    };
+    }
 
     public void hideKeyboard(){
         ((InputMethodManager) VLCApplication.getAppContext().getSystemService(INPUT_METHOD_SERVICE)).hideSoftInputFromWindow(
