@@ -39,7 +39,6 @@ import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.audio.MediaComparators;
 import org.videolan.vlc.util.CustomDirectories;
-import org.videolan.vlc.util.Strings;
 import org.videolan.vlc.util.Util;
 
 import java.io.File;
@@ -116,8 +115,10 @@ public class BaseBrowserAdapter extends  RecyclerView.Adapter<RecyclerView.ViewH
             @Override
             public void onClick(View v) {
                 MediaWrapper mw = (MediaWrapper) getItem(holder.getAdapterPosition());
+                mw.removeFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
+
                 if (mw.getType() == MediaWrapper.TYPE_DIR)
-                    fragment.browse(mw, holder.getAdapterPosition());
+                    fragment.browse(mw, holder.getAdapterPosition(), true);
                 else if (mw.getType() == MediaWrapper.TYPE_VIDEO)
                     Util.openMedia(v.getContext(), mw);
                 else  if (mw.getType() == MediaWrapper.TYPE_AUDIO) {
@@ -188,13 +189,13 @@ public class BaseBrowserAdapter extends  RecyclerView.Adapter<RecyclerView.ViewH
     }
 
     public static class Storage {
-        String path;
+        Uri uri;
         String name;
         String description;
 
-        public Storage(String location){
-            path = location;
-            name = Strings.getName(path);
+        public Storage(Uri uri){
+            this.uri = uri;
+            name = uri.getLastPathSegment();
         }
 
         public String getName() {
@@ -213,8 +214,8 @@ public class BaseBrowserAdapter extends  RecyclerView.Adapter<RecyclerView.ViewH
             return description;
         }
 
-        public String getPath() {
-            return path;
+        public Uri getUri() {
+            return uri;
         }
     }
 
@@ -321,7 +322,7 @@ public class BaseBrowserAdapter extends  RecyclerView.Adapter<RecyclerView.ViewH
         notifyDataSetChanged();
     }
 
-    private int getIconResId(MediaWrapper media) {
+    protected int getIconResId(MediaWrapper media) {
         switch (media.getType()){
             case MediaWrapper.TYPE_AUDIO:
                 return R.drawable.ic_browser_audio_normal;
