@@ -31,9 +31,10 @@ import android.content.res.Resources;
 import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
+import android.os.Message;
 import android.preference.PreferenceManager;
+import android.support.design.widget.Snackbar;
 import android.support.v4.app.FragmentActivity;
-import android.support.v7.app.AlertDialog;
 import android.util.DisplayMetrics;
 import android.util.Log;
 import android.view.ContextMenu;
@@ -53,16 +54,15 @@ import android.widget.PopupMenu;
 import android.widget.PopupMenu.OnMenuItemClickListener;
 import android.widget.TextView;
 
-import org.videolan.libvlc.LibVlcUtil;
 import org.videolan.libvlc.Media;
+import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.vlc.MediaDatabase;
 import org.videolan.vlc.MediaGroup;
 import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.MediaWrapper;
 import org.videolan.vlc.R;
 import org.videolan.vlc.Thumbnailer;
-import org.videolan.vlc.audio.AudioServiceController;
-import org.videolan.vlc.gui.CommonDialogs;
+import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.gui.SecondaryActivity;
 import org.videolan.vlc.gui.browser.MediaBrowserFragment;
@@ -71,9 +71,10 @@ import org.videolan.vlc.interfaces.IVideoBrowser;
 import org.videolan.vlc.util.AndroidDevices;
 import org.videolan.vlc.util.Util;
 import org.videolan.vlc.util.VLCInstance;
-import org.videolan.vlc.util.VLCRunnable;
+import org.videolan.vlc.util.WeakHandler;
 import org.videolan.vlc.widget.SwipeRefreshLayout;
 
+import java.io.File;
 import java.util.HashMap;
 import java.util.List;
 import java.util.concurrent.BrokenBarrierException;
@@ -85,6 +86,9 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
 
     public final static String KEY_GROUP = "key_group";
 
+    private static final int DELETE_MEDIA = 0;
+    private static final int DELETE_DURATION = 3000;
+
     protected static final String ACTION_SCAN_START = "org.videolan.vlc.gui.ScanStart";
     protected static final String ACTION_SCAN_STOP = "org.videolan.vlc.gui.ScanStop";
     protected static final int UPDATE_ITEM = 0;
@@ -108,7 +112,6 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     private VideoGridAnimator mAnimator;
 
     private MainActivity mMainActivity;
-    private AudioServiceController mAudioController;
 
     // Gridview position saved in onPause()
     private int mGVFirstVisiblePos;
@@ -120,8 +123,6 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
 
-        mAudioController = AudioServiceController.getInstance();
-
         mVideoAdapter = new VideoListAdapter(this);
         mMediaLibrary = MediaLibrary.getInstance();
 
@@ -132,6 +133,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         if (activity != null)
             mThumbnailer = new Thumbnailer(activity, activity.getWindowManager().getDefaultDisplay());
     }
+
     @Override
     public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState){
 
@@ -149,7 +151,8 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
 
         mGridView.setOnScrollListener(new AbsListView.OnScrollListener() {
             @Override
-            public void onScrollStateChanged(AbsListView view, int scrollState) {}
+            public void onScrollStateChanged(AbsListView view, int scrollState) {
+            }
 
             @Override
             public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount, int totalItemCount) {
@@ -181,8 +184,6 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     @Override
     public void onPause() {
         super.onPause();
-        if (!(getActivity() instanceof MainActivity))
-            AudioServiceController.getInstance().unbindAudioService(getActivity());
         mGVFirstVisiblePos = mGridView.getFirstVisiblePosition();
         mMediaLibrary.setBrowser(null);
         mMediaLibrary.removeUpdateHandler(mHandler);
@@ -195,9 +196,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     @Override
     public void onResume() {
         super.onResume();
-        if (!(getActivity() instanceof MainActivity))
-            AudioServiceController.getInstance().bindAudioService(getActivity());
-        else
+        if ((getActivity() instanceof MainActivity))
             mMainActivity = (MainActivity) getActivity();
         mMediaLibrary.setBrowser(this);
         mMediaLibrary.addUpdateHandler(mHandler);
@@ -209,7 +208,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
             focusHelper(false);
         }
         //Get & set times
-        HashMap<String, Long> times = MediaDatabase.getInstance().getVideoTimes(getActivity());
+        HashMap<String, Long> times = MediaDatabase.getInstance().getVideoTimes();
         mVideoAdapter.setTimes(times);
         mGridView.setSelection(mGVFirstVisiblePos);
         updateViewMode();
@@ -298,11 +297,15 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     }
 
     protected void playVideo(MediaWrapper media, boolean fromStart) {
-        VideoPlayerActivity.start(getActivity(), media.getLocation(), fromStart);
+        media.removeFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
+        VideoPlayerActivity.start(getActivity(), media.getUri(), fromStart);
     }
 
     protected void playAudio(MediaWrapper media) {
-        mAudioController.load(media, true);
+        if (mService != null) {
+            media.addFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
+            mService.load(media);
+        }
     }
 
     private boolean handleContextItemSelected(MenuItem menu, int position) {
@@ -330,20 +333,11 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
                 }
                 return true;
             case R.id.video_list_delete:
-                AlertDialog alertDialog = CommonDialogs.deleteMedia(
-                        getActivity(),
-                        media.getLocation(),
-                        new VLCRunnable(media) {
-                            @Override
-                            public void run(Object o) {
-                                MediaWrapper media = (MediaWrapper) o;
-                                mMediaLibrary.getMediaItems().remove(media);
-                                mVideoAdapter.remove(media);
-                                if (mAudioController.getMediaLocations().contains(media.getLocation()))
-                                    mAudioController.removeLocation(media.getLocation());
-                            }
-                        });
-                alertDialog.show();
+                Snackbar.make(getView(), getString(R.string.file_deleted), Snackbar.LENGTH_LONG)
+                    .setAction(android.R.string.cancel, mCancelDeleteMediaListener)
+                    .show();
+                Message msg = mDeleteHandler.obtainMessage(DELETE_MEDIA, position, 0);
+                mDeleteHandler.sendMessageDelayed(msg, DELETE_DURATION);
                 return true;
         }
         return false;
@@ -367,13 +361,13 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
             menu.findItem(R.id.video_list_play_from_start).setVisible(true);
 
         boolean hasInfo = false;
-        final Media media = new Media(VLCInstance.get(), mediaWrapper.getLocation());
+        final Media media = new Media(VLCInstance.get(), mediaWrapper.getUri());
         media.parse();
-        media.release();
         if (media.getMeta(Media.Meta.Title) != null)
             hasInfo = true;
+        media.release();
         menu.findItem(R.id.video_list_info).setVisible(hasInfo);
-        menu.findItem(R.id.video_list_delete).setVisible(!LibVlcUtil.isLolliPopOrLater() ||
+        menu.findItem(R.id.video_list_delete).setVisible(!AndroidUtil.isLolliPopOrLater() ||
                 mediaWrapper.getLocation().startsWith("file://" + AndroidDevices.EXTERNAL_PUBLIC_DIRECTORY));
     }
 
@@ -387,7 +381,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
 
     @TargetApi(Build.VERSION_CODES.HONEYCOMB)
     public void onContextPopupMenu(View anchor, final int position) {
-        if (!LibVlcUtil.isHoneycombOrLater()) {
+        if (!AndroidUtil.isHoneycombOrLater()) {
             // Call the "classic" context menu
             anchor.performLongClick();
             return;
@@ -558,7 +552,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
             getActivity().runOnUiThread(new Runnable() {
                 @Override
                 public void run() {
-                    mViewNomedia.setVisibility(mVideoAdapter.getCount()>0 ? View.GONE : View.VISIBLE);
+                    mViewNomedia.setVisibility(mVideoAdapter.getCount() > 0 ? View.GONE : View.VISIBLE);
                     mReadyToDisplay = true;
                     mVideoAdapter.setNotifyOnChange(true);
                     mVideoAdapter.sort();
@@ -573,4 +567,48 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     public void clear(){
         mVideoAdapter.clear();
     }
+
+    public void deleteMedia(int position){
+        final MediaWrapper media = mVideoAdapter.getItem(position);
+        final String path = media.getUri().getPath();
+        new Thread(new Runnable() {
+            public void run() {
+                Util.recursiveDelete(VLCApplication.getAppContext(), new File(path));
+            }
+        }).start();
+        mMediaLibrary.getMediaItems().remove(media);
+        mVideoAdapter.remove(media);
+        if (mService != null) {
+            final List<String> list = mService.getMediaLocations();
+            if (list != null && list.contains(media.getLocation())) {
+                mService.removeLocation(media.getLocation());
+            }
+        }
+    }
+
+
+    View.OnClickListener mCancelDeleteMediaListener = new View.OnClickListener() {
+        @Override
+        public void onClick(View view) {
+            mDeleteHandler.removeMessages(DELETE_MEDIA);
+        }
+    };
+
+    Handler mDeleteHandler = new VideoDeleteHandler(this);
+
+    private static class VideoDeleteHandler extends WeakHandler<VideoGridFragment>{
+
+        public VideoDeleteHandler(VideoGridFragment owner) {
+            super(owner);
+        }
+
+        @Override
+        public void handleMessage(Message msg) {
+            super.handleMessage(msg);
+            switch (msg.what){
+                case DELETE_MEDIA:
+                    getOwner().deleteMedia(msg.arg1);
+            }
+        }
+    }
 }
