@@ -95,10 +95,9 @@ public class BitmapUtil {
                  * serving the file from the database and
                  * adding it to the memcache for later use.
                  */
-                Context c = VLCApplication.getAppContext();
                 picture = readCoverBitmap(media.getArtworkURL());
                 if (picture == null)
-                    picture = MediaDatabase.getInstance().getPicture(c, media.getLocation());
+                    picture = MediaDatabase.getInstance().getPicture(media.getUri());
                 cache.addBitmapToMemCache(media.getLocation(), picture);
             }
             return picture;
