@@ -20,36 +20,13 @@
  *****************************************************************************/
 package org.videolan.vlc.gui.tv;
 
-import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.concurrent.BrokenBarrierException;
-import java.util.concurrent.CyclicBarrier;
-
-import org.videolan.vlc.MediaDatabase;
-import org.videolan.vlc.MediaLibrary;
-import org.videolan.vlc.MediaWrapper;
-import org.videolan.vlc.R;
-import org.videolan.vlc.Thumbnailer;
-import org.videolan.vlc.gui.PreferencesActivity;
-import org.videolan.vlc.gui.tv.audioplayer.AudioPlayerActivity;
-import org.videolan.vlc.gui.tv.browser.MusicFragment;
-import org.videolan.vlc.gui.tv.browser.VerticalGridActivity;
-import org.videolan.vlc.interfaces.IVideoBrowser;
-import org.videolan.vlc.gui.video.VideoListHandler;
-import org.videolan.vlc.util.AndroidDevices;
-import org.videolan.vlc.util.VLCInstance;
-
 import android.app.Activity;
 import android.app.FragmentManager;
-import android.content.BroadcastReceiver;
-import android.content.Context;
 import android.content.Intent;
-import android.content.IntentFilter;
 import android.graphics.Bitmap;
 import android.graphics.drawable.Drawable;
-import android.net.ConnectivityManager;
-import android.net.NetworkInfo;
 import android.os.AsyncTask;
+import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
 import android.support.v17.leanback.app.BackgroundManager;
@@ -68,7 +45,28 @@ import android.view.View;
 import android.view.View.OnClickListener;
 import android.widget.ProgressBar;
 
-public class MainTvActivity extends Activity implements IVideoBrowser, OnItemViewSelectedListener,
+import org.videolan.vlc.MediaDatabase;
+import org.videolan.vlc.MediaLibrary;
+import org.videolan.vlc.MediaWrapper;
+import org.videolan.vlc.PlaybackService;
+import org.videolan.vlc.R;
+import org.videolan.vlc.Thumbnailer;
+import org.videolan.vlc.gui.PreferencesActivity;
+import org.videolan.vlc.gui.tv.audioplayer.AudioPlayerActivity;
+import org.videolan.vlc.gui.tv.browser.BaseTvActivity;
+import org.videolan.vlc.gui.tv.browser.MusicFragment;
+import org.videolan.vlc.gui.tv.browser.VerticalGridActivity;
+import org.videolan.vlc.gui.video.VideoListHandler;
+import org.videolan.vlc.interfaces.IVideoBrowser;
+import org.videolan.vlc.util.AndroidDevices;
+import org.videolan.vlc.util.VLCInstance;
+
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.concurrent.BrokenBarrierException;
+import java.util.concurrent.CyclicBarrier;
+
+public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnItemViewSelectedListener,
         OnItemViewClickedListener, OnClickListener {
 
     private static final int NUM_ITEMS_PREVIEW = 5;
@@ -87,7 +85,6 @@ public class MainTvActivity extends Activity implements IVideoBrowser, OnItemVie
     protected BrowseFragment mBrowseFragment;
     private ProgressBar mProgressBar;
     protected final CyclicBarrier mBarrier = new CyclicBarrier(2);
-    private MediaLibrary mMediaLibrary;
     private static Thumbnailer sThumbnailer;
     private MediaWrapper mItemToUpdate;
     ArrayObjectAdapter mRowsAdapter = new ArrayObjectAdapter(new ListRowPresenter());
@@ -104,25 +101,16 @@ public class MainTvActivity extends Activity implements IVideoBrowser, OnItemVie
     @Override
     protected void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
+        mMediaLibrary.loadMediaItems(false);
 
         if (!VLCInstance.testCompatibleCPU(this)) {
             finish();
             return;
         }
 
-        /*
-         * skip browser and show directly Audio Player if a song is playing
-         */
-        if (VLCInstance.get().isPlaying()){
-            startActivity(new Intent(this, AudioPlayerActivity.class));
-            finish();
-            return;
-        }
-
         mContext = this;
         setContentView(R.layout.tv_main_fragment);
 
-        mMediaLibrary = MediaLibrary.getInstance();
         mDefaultBackground = getResources().getDrawable(R.drawable.background);
         final FragmentManager fragmentManager = getFragmentManager();
         mBrowseFragment = (BrowseFragment) fragmentManager.findFragmentById(
@@ -133,17 +121,29 @@ public class MainTvActivity extends Activity implements IVideoBrowser, OnItemVie
         mBrowseFragment.setHeadersState(BrowseFragment.HEADERS_ENABLED);
         mBrowseFragment.setTitle(getString(R.string.app_name));
         mBrowseFragment.setBadgeDrawable(getResources().getDrawable(R.drawable.icon));
-        // set search icon color
-        mBrowseFragment.setSearchAffordanceColor(getResources().getColor(R.color.orange500));
 
         // add a listener for selected items
         mBrowseFragment.setOnItemViewClickedListener(this);
         mBrowseFragment.setOnItemViewSelectedListener(this);
 
-        mBrowseFragment.setOnSearchClickedListener(this);
+        if (!Build.MANUFACTURER.equalsIgnoreCase("amazon")) { //Hide search for Amazon Fire TVs
+            mBrowseFragment.setOnSearchClickedListener(this);
+            // set search icon color
+            mBrowseFragment.setSearchAffordanceColor(getResources().getColor(R.color.orange500));
+        }
         mRootContainer = mBrowseFragment.getView();
-        mMediaLibrary.loadMediaItems(this, true);
-        BackgroundManager.getInstance(this).attach(getWindow());
+    }
+
+    @Override
+    public void onConnected(PlaybackService service) {
+        super.onConnected(service);
+        /*
+         * skip browser and show directly Audio Player if a song is playing
+         */
+        if (mService.isPlaying()) {
+            startActivity(new Intent(this, AudioPlayerActivity.class));
+            finish();
+        }
     }
 
     @Override
@@ -162,9 +162,8 @@ public class MainTvActivity extends Activity implements IVideoBrowser, OnItemVie
 
         if (mMediaLibrary.isWorking()) //Display UI while MediaLib is scanning
             updateList();
-        //Handle network connection state
-        IntentFilter filter = new IntentFilter(ConnectivityManager.CONNECTIVITY_ACTION);
-        registerReceiver(networkReceiver, filter);
+
+        mBrowseFragment.setBrandColor(getResources().getColor(R.color.orange800));
     }
 
     protected void onPause() {
@@ -175,7 +174,6 @@ public class MainTvActivity extends Activity implements IVideoBrowser, OnItemVie
         if (sThumbnailer != null)
             sThumbnailer.setVideoBrowser(null);
         mBarrier.reset();
-        unregisterReceiver(networkReceiver);
     }
 
     @Override
@@ -190,7 +188,7 @@ public class MainTvActivity extends Activity implements IVideoBrowser, OnItemVie
         super.onActivityResult(requestCode, resultCode, data);
         if (requestCode == ACTIVITY_RESULT_PREFERENCES) {
             if (resultCode == PreferencesActivity.RESULT_RESCAN)
-                MediaLibrary.getInstance().loadMediaItems(this, true);
+                MediaLibrary.getInstance().loadMediaItems(true);
             else if (resultCode == PreferencesActivity.RESULT_RESTART) {
                 Intent intent = getIntent();
                 finish();
@@ -201,7 +199,7 @@ public class MainTvActivity extends Activity implements IVideoBrowser, OnItemVie
 
     @Override
     public boolean onKeyDown(int keyCode, KeyEvent event) {
-        if ((keyCode == KeyEvent.KEYCODE_MEDIA_PLAY_PAUSE || keyCode == KeyEvent.KEYCODE_BUTTON_Y) && mSelectedItem instanceof MediaWrapper){
+        if ((keyCode == KeyEvent.KEYCODE_MEDIA_PLAY_PAUSE || keyCode == KeyEvent.KEYCODE_BUTTON_Y) && mSelectedItem instanceof MediaWrapper) {
             MediaWrapper media = (MediaWrapper) mSelectedItem;
             if (media.getType() != MediaWrapper.TYPE_DIR)
                 return false;
@@ -263,7 +261,8 @@ public class MainTvActivity extends Activity implements IVideoBrowser, OnItemVie
     }
 
     @Override
-    public void sendTextInfo(String info, int progress, int max) {}
+    public void sendTextInfo(String info, int progress, int max) {
+    }
 
     @Override
     public void setItemToUpdate(MediaWrapper item) {
@@ -280,7 +279,8 @@ public class MainTvActivity extends Activity implements IVideoBrowser, OnItemVie
         try {
             mBarrier.await();
         } catch (InterruptedException e) {
-        } catch (BrokenBarrierException e) {}
+        } catch (BrokenBarrierException e) {
+        }
 
     }
 
@@ -293,8 +293,8 @@ public class MainTvActivity extends Activity implements IVideoBrowser, OnItemVie
 
     @Override
     public void onItemClicked(Presenter.ViewHolder itemViewHolder, Object item, RowPresenter.ViewHolder rowViewHolder, Row row) {
-        if (row.getId() == HEADER_CATEGORIES){
-            CardPresenter.SimpleCard card = (CardPresenter.SimpleCard)item;
+        if (row.getId() == HEADER_CATEGORIES) {
+            CardPresenter.SimpleCard card = (CardPresenter.SimpleCard) item;
             Intent intent = new Intent(mContext, VerticalGridActivity.class);
             intent.putExtra(BROWSER_TYPE, HEADER_CATEGORIES);
             intent.putExtra(MusicFragment.AUDIO_CATEGORY, card.getId());
@@ -303,7 +303,7 @@ public class MainTvActivity extends Activity implements IVideoBrowser, OnItemVie
             TvUtil.openMedia(mContext, item, row);
         else if (row.getId() == HEADER_MISC)
             startActivityForResult(new Intent(this, PreferencesActivity.class), ACTIVITY_RESULT_PREFERENCES);
-        else if (row.getId() == HEADER_NETWORK){
+        else if (row.getId() == HEADER_NETWORK) {
             TvUtil.openMedia(mContext, item, row);
         }
     }
@@ -316,10 +316,11 @@ public class MainTvActivity extends Activity implements IVideoBrowser, OnItemVie
 
     public class AsyncUpdate extends AsyncTask<Void, Void, Void> {
 
-        public AsyncUpdate() { }
+        public AsyncUpdate() {
+        }
 
         @Override
-        protected void onPreExecute(){
+        protected void onPreExecute() {
             mRowsAdapter.clear();
             mProgressBar.setVisibility(View.VISIBLE);
 
@@ -403,9 +404,9 @@ public class MainTvActivity extends Activity implements IVideoBrowser, OnItemVie
                 ArrayList<MediaWrapper> videoList = mMediaLibrary.getVideoItems();
                 MediaDatabase mediaDatabase = MediaDatabase.getInstance();
                 if (sThumbnailer != null && videoList != null && !videoList.isEmpty()) {
-                    for (MediaWrapper MediaWrapper : videoList){
-                        picture = mediaDatabase.getPicture(mContext, MediaWrapper.getLocation());
-                        if (picture== null)
+                    for (MediaWrapper MediaWrapper : videoList) {
+                        picture = mediaDatabase.getPicture(MediaWrapper.getUri());
+                        if (picture == null)
                             sThumbnailer.addJob(MediaWrapper);
                     }
                     if (sThumbnailer.getJobsCount() > 0)
@@ -415,32 +416,11 @@ public class MainTvActivity extends Activity implements IVideoBrowser, OnItemVie
         }).start();
     }
 
-    public static Thumbnailer getThumbnailer(){
+    public static Thumbnailer getThumbnailer() {
         return sThumbnailer;
     }
 
-    private final BroadcastReceiver networkReceiver = new BroadcastReceiver() {
-        boolean connected = true;
-        @Override
-        public void onReceive(Context context, Intent intent) {
-            if (mMediaLibrary.isWorking())
-                return;
-            String action = intent.getAction();
-            if (ConnectivityManager.CONNECTIVITY_ACTION.equals(action)) {
-                final NetworkInfo networkInfo = ((ConnectivityManager) VLCApplication.getAppContext().getSystemService(
-                        Context.CONNECTIVITY_SERVICE)).getActiveNetworkInfo();
-                if (networkInfo == null || networkInfo.getState() == NetworkInfo.State.CONNECTED) {
-                    if (networkInfo == null){
-                        if (connected)
-                            connected = false;
-                        else
-                            return; //block consecutive calls when disconnected
-                    } else
-                        connected = true;
-                    updateList();
-                }
-
-            }
-        }
-    };
+    protected void refresh() {
+        mMediaLibrary.loadMediaItems(true);
+    }
 }
