@@ -23,20 +23,25 @@ package org.videolan.vlc.gui.tv.audioplayer;
 import java.util.ArrayList;
 import java.util.Collections;
 
+import org.videolan.libvlc.Media;
+import org.videolan.libvlc.MediaPlayer;
 import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.MediaWrapper;
+import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
-import org.videolan.vlc.audio.AudioServiceController;
-import org.videolan.vlc.audio.RepeatType;
 import org.videolan.vlc.gui.DividerItemDecoration;
+import org.videolan.vlc.gui.PlaybackServiceActivity;
+import org.videolan.vlc.gui.PreferencesActivity;
 import org.videolan.vlc.gui.audio.AudioUtil;
 import org.videolan.vlc.gui.audio.MediaComparators;
-import org.videolan.vlc.interfaces.IAudioPlayer;
+import org.videolan.vlc.gui.tv.browser.BaseTvActivity;
 import org.videolan.vlc.util.AndroidDevices;
+import org.videolan.vlc.util.Util;
 
-import android.app.Activity;
+import android.content.SharedPreferences;
 import android.graphics.Bitmap;
 import android.os.Bundle;
+import android.preference.PreferenceManager;
 import android.support.v7.widget.LinearLayoutManager;
 import android.support.v7.widget.RecyclerView;
 import android.view.InputDevice;
@@ -47,12 +52,12 @@ import android.widget.ImageView;
 import android.widget.ProgressBar;
 import android.widget.TextView;
 
-public class AudioPlayerActivity extends Activity implements AudioServiceController.AudioServiceConnectionListener, IAudioPlayer, View.OnFocusChangeListener {
+public class AudioPlayerActivity extends BaseTvActivity implements PlaybackService.Client.Callback,
+        PlaybackService.Callback, View.OnFocusChangeListener {
     public static final String TAG = "VLC/AudioPlayerActivity";
 
     public static final String MEDIA_LIST = "media_list";
 
-    private AudioServiceController mAudioController;
     private RecyclerView mRecyclerView;
     private PlaylistAdapter mAdapter;
     private LinearLayoutManager mLayoutManager;
@@ -84,9 +89,6 @@ public class AudioPlayerActivity extends Activity implements AudioServiceControl
         mAdapter = new PlaylistAdapter(this, mMediaList);
         mRecyclerView.setAdapter(mAdapter);
 
-        mAudioController = AudioServiceController.getInstance();
-
-        mAudioController.getRepeatType();
         mTitleTv = (TextView)findViewById(R.id.media_title);
         mArtistTv = (TextView)findViewById(R.id.media_artist);
         mNext = (ImageView)findViewById(R.id.button_next);
@@ -98,17 +100,13 @@ public class AudioPlayerActivity extends Activity implements AudioServiceControl
         findViewById(R.id.button_shuffle).setOnFocusChangeListener(this);
     }
 
-    public void onStart(){
-        super.onStart();
-        mAudioController.bindAudioService(this, this);
-        mAudioController.addAudioPlayer(this);
-    }
 
-    public void onStop(){
+    @Override
+    protected void onStop() {
+        /* unregister before super.onStop() since mService is set to null from this call */
+        if (mService != null)
+            mService.removeCallback(this);
         super.onStop();
-        mAudioController.removeAudioPlayer(this);
-        mAudioController.unbindAudioService(this);
-        mMediaList.clear();
     }
 
     protected void onResume() {
@@ -122,10 +120,13 @@ public class AudioPlayerActivity extends Activity implements AudioServiceControl
     };
 
     @Override
-    public void onConnectionSuccess() {
-        ArrayList<MediaWrapper> medias = (ArrayList<MediaWrapper>) mAudioController.getMedias();
+    public void onConnected(PlaybackService service) {
+        super.onConnected(service);
+
+        mService.addCallback(this);
+        ArrayList<MediaWrapper> medias = (ArrayList<MediaWrapper>) mService.getMedias();
         if (!mMediaList.isEmpty() && !mMediaList.equals(medias)) {
-            mAudioController.load(mMediaList, 0);
+            mService.load(mMediaList, 0);
         } else {
             mMediaList = medias;
             update();
@@ -135,19 +136,35 @@ public class AudioPlayerActivity extends Activity implements AudioServiceControl
     }
 
     @Override
-    public void onConnectionFailed() {}
+    protected void refresh() {
+        update();
+    }
+
+    @Override
+    protected void updateList() {
+        update();
+    }
 
     @Override
     public void update() {
-        mPlayPauseButton.setImageResource(mAudioController.isPlaying() ? R.drawable.ic_pause : R.drawable.ic_play);
-        if (mAudioController.hasMedia()) {
-            mTitleTv.setText(mAudioController.getTitle());
-            mArtistTv.setText(mAudioController.getArtist());
-            mProgressBar.setMax(mAudioController.getLength());
-            MediaWrapper MediaWrapper = MediaLibrary.getInstance().getMediaItem(mAudioController.getCurrentMediaLocation());
+        if (mService == null)
+            return;
+        mPlayPauseButton.setImageResource(mService.isPlaying() ? R.drawable.ic_pause : R.drawable.ic_play);
+        if (mService.hasMedia()) {
+            SharedPreferences mSettings= PreferenceManager.getDefaultSharedPreferences(this);
+            if (mSettings.getBoolean(PreferencesActivity.VIDEO_RESTORE, false)){
+                Util.commitPreferences(mSettings.edit().putBoolean(PreferencesActivity.VIDEO_RESTORE, false));
+                mService.switchToVideo();
+                finish();
+                return;
+            }
+            mTitleTv.setText(mService.getTitle());
+            mArtistTv.setText(mService.getArtist());
+            mProgressBar.setMax((int) mService.getLength());
+            MediaWrapper MediaWrapper = MediaLibrary.getInstance().getMediaItem(mService.getCurrentMediaLocation());
             Bitmap cover = AudioUtil.getCover(this, MediaWrapper, mCover.getWidth());
             if (cover == null)
-                cover = mAudioController.getCover();
+                cover = mService.getCover();
             if (cover == null)
                 mCover.setImageResource(R.drawable.background_cone);
             else
@@ -157,7 +174,18 @@ public class AudioPlayerActivity extends Activity implements AudioServiceControl
 
     @Override
     public void updateProgress() {
-        mProgressBar.setProgress(mAudioController.getTime());
+        if (mService != null)
+            mProgressBar.setProgress((int)mService.getTime());
+    }
+
+    @Override
+    public void onMediaEvent(Media.Event event) {
+
+    }
+
+    @Override
+    public void onMediaPlayerEvent(MediaPlayer.Event event) {
+
     }
 
     public boolean onKeyDown(int keyCode, KeyEvent event){
@@ -173,6 +201,7 @@ public class AudioPlayerActivity extends Activity implements AudioServiceControl
                 return true;
             case KeyEvent.KEYCODE_F:
             case KeyEvent.KEYCODE_BUTTON_R1:
+            case KeyEvent.KEYCODE_MEDIA_NEXT:
                 goNext();
                 return true;
             case KeyEvent.KEYCODE_MEDIA_FAST_FORWARD:
@@ -183,6 +212,7 @@ public class AudioPlayerActivity extends Activity implements AudioServiceControl
                 return true;
             case KeyEvent.KEYCODE_R:
             case KeyEvent.KEYCODE_BUTTON_L1:
+            case KeyEvent.KEYCODE_MEDIA_PREVIOUS:
                 goPrevious();
                 return true;
             /*
@@ -208,7 +238,9 @@ public class AudioPlayerActivity extends Activity implements AudioServiceControl
     }
 
     public void playSelection() {
-        mAudioController.playIndex(mAdapter.getmSelectedItem());
+        if (mService == null)
+            return;
+        mService.playIndex(mAdapter.getmSelectedItem());
         mCurrentlyPlaying = mAdapter.getmSelectedItem();
     }
 
@@ -240,10 +272,12 @@ public class AudioPlayerActivity extends Activity implements AudioServiceControl
     }
 
     private void seek(int delta) {
-        int time = mAudioController.getTime()+delta;
-        if (time < 0 || time > mAudioController.getLength())
+        if (mService == null)
             return;
-        mAudioController.setTime(time);
+        int time = (int) mService.getTime()+delta;
+        if (time < 0 || time > mService.getLength())
+            return;
+        mService.setTime(time);
     }
 
     public void onClick(View v){
@@ -267,53 +301,59 @@ public class AudioPlayerActivity extends Activity implements AudioServiceControl
     }
 
     private void setShuffleMode(boolean shuffle) {
+        if (mService == null)
+            return;
         mShuffling = shuffle;
         mShuffle.setImageResource(shuffle ? R.drawable.ic_shuffle_on :
                 R.drawable.ic_shuffle);
-        ArrayList<MediaWrapper> medias = (ArrayList<MediaWrapper>) mAudioController.getMedias();
+        ArrayList<MediaWrapper> medias = (ArrayList<MediaWrapper>) mService.getMedias();
         if (shuffle){
             Collections.shuffle(medias);
         } else {
             Collections.sort(medias, MediaComparators.byTrackNumber);
         }
-        mAudioController.load(medias, 0);
+        mService.load(medias, 0);
         mAdapter.updateList(medias);
         update();
     }
 
     private void updateRepeatMode() {
-        RepeatType type = mAudioController.getRepeatType();
-        if (type == RepeatType.None){
-            mAudioController.setRepeatType(RepeatType.All);
+        if (mService == null)
+            return;
+        PlaybackService.RepeatType type = mService.getRepeatType();
+        if (type == PlaybackService.RepeatType.None){
+            mService.setRepeatType(PlaybackService.RepeatType.All);
             mRepeat.setImageResource(R.drawable.ic_repeat_on);
-        } else if (type == RepeatType.All) {
-            mAudioController.setRepeatType(RepeatType.Once);
+        } else if (type == PlaybackService.RepeatType.All) {
+            mService.setRepeatType(PlaybackService.RepeatType.Once);
             mRepeat.setImageResource(R.drawable.ic_repeat_one);
-        } else if (type == RepeatType.Once) {
-            mAudioController.setRepeatType(RepeatType.None);
+        } else if (type == PlaybackService.RepeatType.Once) {
+            mService.setRepeatType(PlaybackService.RepeatType.None);
             mRepeat.setImageResource(R.drawable.ic_repeat);
         }
     }
 
     private void goPrevious() {
-        if (mAudioController.hasPrevious()) {
-            mAudioController.previous();
+        if (mService != null && mService.hasPrevious()) {
+            mService.previous();
             selectItem(--mCurrentlyPlaying);
         }
     }
 
     private void goNext() {
-        if (mAudioController.hasNext()){
-            mAudioController.next();
+        if (mService != null && mService.hasNext()){
+            mService.next();
             selectItem(++mCurrentlyPlaying);
         }
     }
 
     private void togglePlayPause() {
-        if (mAudioController.isPlaying())
-            mAudioController.pause();
-        else if (mAudioController.hasMedia())
-            mAudioController.play();
+        if (mService == null)
+            return;
+        if (mService.isPlaying())
+            mService.pause();
+        else if (mService.hasMedia())
+            mService.play();
     }
 
     private void selectNext() {
